function LL = computeLL(communityStatistics)
%computeLL Computes the log-likelihood of a community with given statistics
observed1sInside = communityStatistics(1);
observed1sOutside = communityStatistics(2);
expectedAreaInside = communityStatistics(3);
expectedAreaOutside = communityStatistics(4);

observed0sInside = expectedAreaInside - observed1sInside;
observed0sOutside = expectedAreaOutside - observed1sOutside;

% Warn if density outside is above the density inside
if any(observed1sInside./expectedAreaInside < observed1sOutside./expectedAreaOutside)
    warning('The density outside the community is higher than inside the community');
end

expect1observe1 = logMult(observed1sInside,observed1sInside/expectedAreaInside);
expect1observe0 = logMult(observed0sInside,observed0sInside/expectedAreaInside);
expect0observe1 = logMult(observed1sOutside,observed1sOutside/expectedAreaOutside);
expect0observe0 = logMult(observed0sOutside,observed0sOutside/expectedAreaOutside);

LL =  expect1observe1 + expect1observe0 + expect0observe1 + expect0observe0;
end
