function [ x, sigma, LL ] = recoverSigmaExhaustivelyFast( comm, specialX, graph, coveredByOthers, partialCoverGraph )
%recoverSigmaExhaustivelyFast Returns the parameters x and sigma of a HyCom
%community - that is, x=0.5 for every model.
if (exist('graph', 'var') && exist('coveredByOthers', 'var'))
    globalLL = true;
else
    if (exist('graph', 'var') || exist('coveredByOthers', 'var'))
        error('Either ''graph'' and ''coveredByOthers'' need to be defined or none of them.');
    end
    globalLL = false;
end

A = comm-diag(diag(comm)); %make sure the diagonal is empty
N = length(A);
x = specialX;
if ~(x == 0.5 || x==0)
    error('x can only be 0.5 or 0 for the HyCom model');
end
numberOfEdges=length(find(triu(A)));

edgeValue=zeros(numberOfEdges,1);
[I,J]=find(triu(A));

for edge=1:numberOfEdges
        i=I(edge)-1;
        j=J(edge)-1;
        edgeValue(edge)=(1-x)*(i*j)+(x)*(i+j);   
end
lastCol = N-1;
minSigma =(1-x)*(lastCol*0)+(x)*(lastCol+0);
sigmas=sortrows(edgeValue);       %sort and exclude too small sigmas as
sigmas = sigmas(sigmas>=minSigma);%we want to have the first col completely
if isempty(sigmas)
    sigmas(1) = minSigma;
end

%%
%now that we have all the possible sigmas, we swich to the corresponding
%H,gamma representations as the code is tuned better for them
Hs = nan(1,length(sigmas));
gammas = nan(1,length(sigmas));
p = getPFromX(x);
for s = 1:length(sigmas)
    theta = getThetaFromP(p, sigmas(s));
    Hs(s) = getH(theta,p,N);
    gammas(s) = getGamma(p,Hs(s),N);
end

%%
%the following code is taken over from recoverHGammaExhausively - yes,
%that's not good programming style
observed1sInside = nan(length(Hs),1);
for con = 1:length(Hs)
    H = Hs(con);
    gamma = gammas(con);
    
    % derived from (i+p)*(j+p) <= theta = (N-1+p)*(H+p) to stay with integers 
    gamma2NH = gamma*gamma-(N-1)*H;
    NH2gamma = N-1+H - 2*gamma;
    thetaPrime = ((N-1)*NH2gamma+gamma2NH)*(H*NH2gamma+gamma2NH);
    edgeValue=((I-1)*NH2gamma+gamma2NH).*((J-1)*NH2gamma+gamma2NH);   
    observed1sInside(con) = length(find(edgeValue <= thetaPrime));
end
%expectedAreaInside2 = arrayfun(@(H,gamma) getAreaOfExpectedCommunity(N,H,gamma, @conditionHGamma), parameterConfig(:,1), parameterConfig(:,2));
%expectedAreaInside = transpose(arrayfun(@(H,gamma) getElementsBelowHGamma(N,H,gamma), Hs, gammas));
expectedAreaInside = transpose(getElementsBelowHGammaParallel(N,Hs,gammas));
observed0sInside = expectedAreaInside - observed1sInside;
if globalLL
    availableArea = getAreaOfTriu(length(graph))- nnz(triu(coveredByOthers));
    availableOnes = length(find(triu(graph))) - nnz(triu(coveredByOthers & graph));
    [expectedAreaOutside, observed1sOutside, observed0sOutside] = globalOutsideDensity(...
             full(double(comm)), N,Hs, gammas, availableArea, availableOnes, ...
             full(double(partialCoverGraph)), expectedAreaInside, observed1sInside);
    
%       l = length(expectedAreaInside);
%       expectedAreaOutside = nan(l,1);
%       observed1sOutside = nan(l,1);
%       observed0sOutside = nan(l,1);
%       for i=1:l
%           [expectedAreaOutside(i), observed1sOutside(i), observed0sOutside(i)] = getGlobalOutsideDensity(...
%               comm, N,Hs(i), gammas(i), availableArea, availableOnes, partialCoverGraph, expectedAreaInside(i), observed1sInside(i));
%       end
%       disp(' ');
else
    expectedAreaOutside = getAreaOfTriu(length(A)) - expectedAreaInside;
    observed1sOutside = numberOfEdges - observed1sInside;
    observed0sOutside = max(0,expectedAreaOutside - observed1sOutside);
end

expect1observe1 = logMult(observed1sInside,observed1sInside./expectedAreaInside);
expect1observe0 = logMult(observed0sInside,observed0sInside./expectedAreaInside);
expect0observe1 = logMult(observed1sOutside,observed1sOutside./expectedAreaOutside);
expect0observe0 = logMult(observed0sOutside,observed0sOutside./expectedAreaOutside);

loglike =  expect1observe1 + expect1observe0 + expect0observe1 + expect0observe0;

if ~isreal(loglike)
    warning('Can''t imagine a loglikelihood! There are %i imaginary loglikelihoods', sum(imag(loglike) ~= 0));
    loglike(imag(loglike) ~= 0) = -Inf;
end

LL = max(loglike);
indicesOfEquivalentParams = find(loglike == LL);
[~,idx] = max(Hs(indicesOfEquivalentParams)); % take the simplest, i.e. the one with max H
H = Hs(indicesOfEquivalentParams(idx));
gamma = gammas(indicesOfEquivalentParams(idx));

%%
% and now, go back to x,sigma representation
p = getP(gamma, N, H);
theta = getTheta(gamma, N, H);
xPrime = getX(p);
if abs(xPrime - x) > 0.001
    fprintf('%f should be %.1f - Is the conversion correct??\n', xPrime, x);
end
sigma = getSigma(theta, p);
end
