function [ x, sigma, LL ] = recoverXSigmaExhaustively( comm, specialX, graph, coveredByOthers, partialCoverGraph )
%recoverXSigmaExhaustively Returns the parameters x and sigma of a community 
if ~exist('specialX', 'var')
    specialX = [];
end    
if (exist('graph', 'var') && exist('coveredByOthers', 'var'))
    globalLL = true;
else
    if (exist('graph', 'var') || exist('coveredByOthers', 'var'))
        error('Either ''graph'' and ''coveredByOthers'' need to be defined or none of them.');
    end
    globalLL = false;
end

A = comm-diag(diag(comm)); %make sure the diagonal is empty
N = length(A);
numberOfEdges=length(find(triu(A)));

edgeValue=zeros(numberOfEdges,1);
[indI,indJ]=find(triu(A));

test=500;
if specialX == 0.5 || specialX == 0 
    % This is the HyCom model (x=0.5 assumes 1-based indexing and x=0 assumes 0-based indexing).
    % We only look for the proper threshold Sigma in this case.
    % Faster version in case of HyCom converts to H,gamma model on the way
    % because the code is better optimized for that.
    if globalLL
        [ x, sigma, LL ] = recoverSigmaExhaustivelyFast(comm, specialX, graph, coveredByOthers, partialCoverGraph );
    else
        [ x, sigma, LL ] = recoverSigmaExhaustivelyFast(comm, specialX);
    end
    return
    
    xToTry = specialX; 
else
    xToTry = [linspace(0.00001,0.99999,test), specialX];
end

result=-inf(length(xToTry),3);
count=1;
for x = xToTry
    for edge=1:numberOfEdges
            i=indI(edge)-1;
            j=indJ(edge)-1;
            edgeValue(edge)=(1-x)*(i*j)+(x)*(i+j);   
    end
    lastCol = N-1;
    minSigma =(1-x)*(lastCol*0)+(x)*(lastCol+0);
    sigmas=sortrows(edgeValue);       %sort and exclude too small sigmas as
    sigmas = sigmas(sigmas>=minSigma);%we want to have the first col completely
    if isempty(sigmas)
        sigmas(1) = minSigma;
    end
    
    %numElementsBelow2 = arrayfun(@(s) getAreaOfExpectedCommunity(N,x,s,@conditionXSigma), sigmas);
    expectedAreaInside = arrayfun(@(s) getElementsBelow(N,x,s), sigmas);
    observed1sInside=((numberOfEdges-length(sigmas)+1):numberOfEdges)';
    observed0sInside=expectedAreaInside-observed1sInside;
    % getElementsBelow might be off by 1 due to the floor function and
    % floating point arithmetics. We work around this by correcting the
    % area if we know that more edges should be inside than the calculated
    % area allows. In case this pragmatic approach does not work in some
    % particular case, the area can be computed alternatively with
    % getAreaOfExpectedCommunity which is much slower but precise.
    numZerosBelowNew=max(0,observed0sInside);
    expectedAreaInside=expectedAreaInside+(numZerosBelowNew-observed0sInside);
    observed0sInside = numZerosBelowNew;
    
    if globalLL
        availableArea = getAreaOfTriu(length(graph))- nnz(triu(coveredByOthers));
        availableOnes = length(find(triu(graph))) - nnz(triu(coveredByOthers & graph));
        l = length(expectedAreaInside);
        expectedAreaOutside = nan(l,1);
        observed1sOutside = nan(l,1);
        observed0sOutside = nan(l,1);
        for i=1:l
            if x~=0.5
                error('The call to getGlobalOutsideDensity might need an update to include all the values of x! Check the H,gamma version.');
            end
            [expectedAreaOutside(i), observed1sOutside(i), observed0sOutside(i)] = getGlobalOutsideDensity(...
                comm, N,x, sigmas(i), availableArea, availableOnes, partialCoverGraph, expectedAreaInside(i), observed1sInside(i)); 
        end
    else
        observed1sOutside=numberOfEdges-observed1sInside;   
        expectedAreaOutside = getAreaOfTriu(N) - expectedAreaInside;
        observed0sOutside = max(0,expectedAreaOutside - observed1sOutside);
    end

    expect1observe1 = logMult(observed1sInside,observed1sInside./expectedAreaInside);
    expect1observe0 = logMult(observed0sInside,observed0sInside./expectedAreaInside);
    expect0observe1 = logMult(observed1sOutside,observed1sOutside./expectedAreaOutside);
    expect0observe0 = logMult(observed0sOutside,observed0sOutside./expectedAreaOutside);

    loglike =  expect1observe1 + expect1observe0 + expect0observe1 + expect0observe0;

    if ~isreal(loglike)
        warning('Can''t imagine a loglikelihood! There are %i imaginary loglikelihoods', sum(imag(loglike) ~= 0));
        loglike(imag(loglike) ~= 0) = -Inf;
    end
    
    [maxLL,~]=max(loglike);
    pos = find(loglike==maxLL,1,'last'); % solve ties by taking the highest sigma
    bestSigma=sigmas(pos);
    result(count,:)=[x,maxLL,bestSigma];
    count=count+1;
end

[LL,index]=max(result(:,2));
x = result(index,1);
sigma = result(index,3);

%if abs(LL - computeLL(getCommunityStatistics(A, x, sigma, @conditionXSigma))) > 1
%    warning('There is something wrong with the LL computation.');
%end
end

function area=getElementsBelow(N, x, sigma)
%only in upper triangular
   cols = 0;
   area = 0;
   for i=0:N
       m = min(N-1,max(0,floor( (sigma - i*x) /  (i - x*(i-1)) )));
       if m <= i
           break;
       else
           area = area + m;
           cols = cols + 1;
       end
   end
   area = area - (cols^2 - cols)/2;
end

function [expectedAreaOutside, observed1sOutside, observed0sOutside] = getGlobalOutsideDensity(...
    comm, N, x, sigma, availableArea, availableOnes, partialCoverGraph, ...
    expectedAreaInside, observed1sInside)
%returns expectedAreaOutside, observed1sOutside, and observed0sOutside for
%given x and sigma but is potentially slow.
    c = makeCommunity( N, x, sigma, @conditionXSigma);
    model = triu(c);
    partialCoverGraphAndModel = nnz(partialCoverGraph & model);
    commAndPartialCoverGraphAndModel = nnz(comm & partialCoverGraph & model);
    expectedAreaOutside = availableArea - expectedAreaInside + partialCoverGraphAndModel;
    observed1sOutside = availableOnes - observed1sInside + commAndPartialCoverGraphAndModel;
    observed0sOutside = expectedAreaOutside - observed1sOutside;
end

